"""Principal Convex Hull Analysis (PCHA) / Archetypal Analysis.

This module is a Python translation of a Matlab module. Avaiable at:
http://www.mortenmorup.dk/MMhomepageUpdated_files/Page327.htm.

Translated by: Ulf Aslak Jensen (contact: ulfjensen@gmail.com)
Original author (Matmab): Morten Morup (contact: morten.morup@gmail.com).
Source paper: 'M. Morup and L. K. Hansen, Archetypal Analysis for Data Mining,
Neural Computing 2011'.
"""

__version__ = "0.1.1"
